from os import path
from pathlib import Path
from typing import Any
from typing import Dict
from typing import Final
from typing import List
from typing import Optional
from typing import Union
from typing import cast
import xml.etree.ElementTree as ET

from eoplatform.console import console


def _not_found_message(*args: str) -> None:

    console.print(
        f"[red bold encircle]:x:  Did not find:[/][yellow] {','.join(list(args))}"
    )

    return None


def extract_metadata(
    file_path: Union[Path, str], target_attributes: Union[List[str], str], **kwargs: Any
) -> Union[Dict[str, Optional[str]], Dict[str, str]]:
    """Extract metadata from file

    Detects filetype and implements the required metadata extractor. Currently supports
    XML and TXT files. Passes additional kwargs to requisite function

    Parameters
    ----------
    file_path : str
        Full file path to target XML file
    target_attributes: Union[List[str], str]
        List of target attributes desired

    Returns
    -------
    Dict[str, Optional[str]]

    """

    file: Path = Path(file_path) if isinstance(file_path, str) else file_path
    file_extension: Optional[str] = file.suffix

    if not file_extension:
        raise ValueError("Input path does not seem to have a file extension")

    file_extension = file_extension.lower()

    if file_extension == ".xml":
        return extract_XML_metadata(file, target_attributes)
    elif file_extension == ".txt":
        return extract_TXT_metadata(file, target_attributes, **kwargs)
    else:
        raise ValueError(f"{file_extension} not currently supported")


def extract_XML_metadata(
    file_path: Union[str, Path], target_attributes: Union[List[str], str]
) -> Dict[str, str]:
    """Extract metadata from XML file

    Uses ElementTree to extract `target_attributes` from `file_path` XML file.
    Verifies that `file_path` exists and is an XML file. Returns dictionary of
    all found attributes

    Parameters
    ----------
    file_path : Union[Path, str]
        Full file path to target XML file
    target_attributes: Union[List[str], str]
        List of target attributes desired

    Returns
    -------
    Dict[str, str]

    """

    X_PATH_WILDCARD: Final[str] = ".//"

    file: Path = Path(file_path) if isinstance(file_path, str) else file_path
    target_attributes_list: List[str] = (
        [target_attributes] if isinstance(target_attributes, str) else target_attributes
    )

    if not file.exists():
        raise FileNotFoundError(f"{file} does not exist")

    file_extension = file.suffix

    if not file_extension:
        raise ValueError("File must have a `.xml` file extension")
    if file_extension.lower() != ".xml":
        raise TypeError(f"{file_path} is not an XML file")

    namespaces: Dict[str, str] = dict(
        [node for _, node in ET.iterparse(file_path, events=["start-ns"])]
    )

    tree: ET.ElementTree = ET.parse(file_path)
    found_attributes: Dict[str, str] = {}

    for target_attribute in target_attributes_list:

        target_el: Optional[ET.Element] = tree.find(
            X_PATH_WILDCARD + target_attribute, namespaces=namespaces
        )

        if target_el is None:
            _not_found_message(target_attribute)
            continue

        found_attributes[target_attribute] = cast(str, target_el.text)

    return found_attributes


def extract_TXT_metadata(
    file_path: Union[Path, str],
    target_attributes: Union[List[str], str],
    delineator: str = "=",
) -> Dict[str, Optional[str]]:
    """Extract metadata from TXT file

    Extracts `target_attributes` from `file_path` TXT file. Assumes metadata
    keys and values are seperated by `delineator`
    Verifies that `file_path` exists and is an TXT file. Returns dictionary of
    all found attributes

    Parameters
    ----------
    file_path : Union[Path, str]
        Full file path to target TXT file
    target_attributes: Union[List[str], str]
        List of target attributes desired

    Returns
    -------
    Dict[str, str]

    """

    file: Path = Path(file_path) if isinstance(file_path, str) else file_path
    target_attributes_list: List[str] = (
        [target_attributes] if isinstance(target_attributes, str) else target_attributes
    )

    if not file.exists():
        raise FileNotFoundError(f"{file_path} does not exist")

    file_extension = file.suffix

    if not file_extension:
        raise ValueError("File must have a `.txt` file extension")
    if file_extension.lower() != ".txt":
        raise TypeError(f"{file_path} is not a TXT file")

    found_attributes: Dict[str, Optional[str]] = {
        k: None for k in target_attributes_list
    }

    with file.open() as f:
        for line_number, line in enumerate(f):

            split: List[str] = line.split(delineator)
            split = [x.strip(" ") for x in split]

            if not len(split) <= 2:
                raise AssertionError(
                    f"Line {line_number} violates formatting assumptions"
                )

            if split[0] not in target_attributes_list:
                continue

            if len(split) != 2:
                raise AssertionError(
                    f"Found {split[0]} on line {line_number} but line does not meet format assumptions"
                )

            found_attributes[split[0]] = split[1].strip("\n")

    if not all(found_attributes.values()):
        not_found: List[str] = list(
            set(target_attributes_list)
            - set(k for k, v in found_attributes.items() if v is not None)
        )
        _not_found_message(*not_found)

    return found_attributes



# type: ignore
import ast
from ast import AST
import math
import operator
from typing import Any
from typing import Dict
from typing import Union


def safe_eval(s, **kwargs) -> Any:
    """Safely parse input formula string

    Modified from Jason M @ SO
    """

    def checkmath(x: str, *args) -> Any:
        if x not in [x for x in dir(math) if not "__" in x]:
            raise SyntaxError(f"Unknown func {x}()")
        fun = getattr(math, x)
        return fun(*args)

    binOps: Dict[type, Union[function, type]] = {
        ast.Add: operator.add,
        ast.Sub: operator.sub,
        ast.Mult: operator.mul,
        ast.Div: operator.truediv,
        ast.Mod: operator.mod,
        ast.Pow: operator.pow,
        ast.Call: checkmath,
        ast.BinOp: ast.BinOp,
    }

    unOps: Dict[type, Union[function, type]] = {
        ast.USub: operator.neg,
        ast.UAdd: operator.pos,
        ast.UnaryOp: ast.UnaryOp,
    }

    ops = tuple(binOps) + tuple(unOps)

    tree: AST = ast.parse(s, mode="eval")

    def _eval(node, **kwargs):
        if isinstance(node, ast.Expression):
            return _eval(node.body, **kwargs)
        elif isinstance(node, ast.Str):
            return node.s
        elif isinstance(node, ast.Num):
            return node.n
        elif isinstance(node, ast.Constant):
            return node.value
        elif isinstance(node, ast.BinOp):
            if isinstance(node.left, ops):
                left = _eval(node.left, **kwargs)
            else:
                left = (
                    kwargs[node.left.id] if hasattr(node.left, "id") else node.right.n
                )
            if isinstance(node.right, ops):
                right = _eval(node.right, **kwargs)
            else:
                right = (
                    kwargs[node.right.id] if hasattr(node.right, "id") else node.right.n
                )
            return binOps[type(node.op)](left, right)
        elif isinstance(node, ast.UnaryOp):
            if isinstance(node.operand, ops):
                operand = _eval(node.operand, **kwargs)
            else:
                operand = node.operand.n
            return unOps[type(node.op)](operand)
        elif isinstance(node, ast.Call):
            args = [_eval(x, **kwargs) for x in node.args]
            r = checkmath(node.func.id, *args)
            return r
        else:
            raise SyntaxError(f"Bad syntax, {type(node)}")

    return _eval(tree, **kwargs)



from dataclasses import dataclass
from dataclasses import field
from typing import Any
from typing import Callable
from typing import Dict
from typing import Final
from typing import Iterator
from typing import Optional
from typing import Sized
from typing import cast

from eoplatform.console import console
from eoplatform.platforms.info.visualizers import InfoVisualizers
from eoplatform.shared_classes import ReturnRender
from rich.panel import Panel


class Base:
    @staticmethod
    def get_meta_unit(object: Any, attribute_name: str) -> str:
        return str(object.__dataclass_fields__[attribute_name].metadata.get("unit", ""))

    def info(self, show_description: bool = True, title: bool = False) -> None:

        VISUALIZERS: Final[Dict[type, Callable[..., ReturnRender]]] = {
            Band: InfoVisualizers.get_band_viz,
            Bands: InfoVisualizers.get_bands_viz,
            Platform: InfoVisualizers.get_platform_viz,
        }

        render: ReturnRender
        for k, v in VISUALIZERS.items():
            if isinstance(self, k):
                render = v(self, show_description=show_description, title=title)
                break

        console.print(
            Panel(
                render.renderable,
                expand=False,
                title=render.title,
                subtitle=render.subtitle,
                width=100,
            )
        )

        return None

    def __str__(self) -> str:
        self.info()
        return ""


@dataclass
class Band(Base):
    number: int
    name: str
    abbreviation: str
    wavelength: Optional[str] = field(metadata={"unit": "um"})
    description: Optional[str]
    sensor: Optional[str]
    resolution: Optional[int] = field(metadata={"unit": "m"})


@dataclass
class Bands(Base):
    def __len__(self) -> int:
        return len(self.__annotations__)

    def __iter__(self) -> Iterator[Band]:
        for band in self.__dict__.values():
            yield band


@dataclass()
class Platform(Base):
    abbreviation: str
    name: str
    var_name: str = field(default="NONE", init=False)
    number_bands: Optional[int] = field(default=None, init=False)
    bands: Optional[Bands] = field(default=None, init=False)

    def __post_init__(self) -> None:
        self.var_name = self.name.replace(" ", "").lower()
        self.number_bands = len(cast(Sized, self.bands)) if self.bands else 0

        return None

    def __len__(self) -> Optional[int]:
        return self.number_bands



from dataclasses import fields
from typing import Any
from typing import Dict
from typing import Final
from typing import List
from typing import cast

from eoplatform.shared_classes import ReturnRender
from rich.console import Group
from rich.table import Table
from rich.text import Text


class CompositeVisualizers:
    @staticmethod
    def get_composite_viz(object: Any) -> ReturnRender:
        """Visualizer for `Composite`"""

        NO_PRINT_ATTRIBUTES: Final[List[str]] = [
            "abbreviation",
            "name",
            "reference",
            "description",
            "formula",
            "type",
            "bands",
        ]

        table = Table(
            show_header=False,
            show_lines=False,
            show_edge=False,
            expand=True,
            row_styles=["", "dim"],
        )

        table.add_column(justify="right", width=30)
        table.add_column(justify="left", width=30)

        table.add_row(
            "Bands",
            ", ".join(list(object.bands)),
        )

        table.add_row(
            "Type",
            object.type.name.capitalize(),
        )

        table.add_row(
            "Formula",
            object.formula,
        )

        field_values: Dict[str, str] = {
            f.name: str(f.default)
            for f in fields(object)
            if f.name not in NO_PRINT_ATTRIBUTES
        }

        field_values = dict(sorted(field_values.items(), key=lambda kv: kv[0]))

        for name, value in field_values.items():

            table.add_row(
                name.capitalize().replace("_", " "),
                f"{value}",
            )

        description: Text = Text("")
        if object.description:
            description = Text(
                cast(str, object.description).strip(), justify="full", end="\n\n"
            )

        group: Group = Group(table, description, fit=True)

        subtitle: str = f"[blue underline][link={object.reference}]Source[/link]"

        title: str = f"[wheat4][bold] {object.name}[/bold] [italic]({object.abbreviation})[/italic]"

        return ReturnRender(renderable=group, title=title, subtitle=subtitle)



from dataclasses import fields
from typing import Any
from typing import Dict
from typing import Final
from typing import List

from eoplatform.shared_classes import ReturnRender
from rich.console import Group
from rich.table import Table
from rich.text import Text


class InfoVisualizers:
    @staticmethod
    def get_band_viz(object: Any, **kwargs: Any) -> ReturnRender:
        """Visualizer for `Band`"""

        table: Table = Table(show_header=False, show_edge=False)
        table.add_column(justify="right")
        table.add_column(justify="left")

        if object.sensor:
            table.add_row("Sensor", f"{object.sensor}")

        if object.wavelength:
            table.add_row(
                "Wavelength",
                f"{object.wavelength} {object.get_meta_unit(object,'wavelength')}",
            )
        if object.resolution:
            table.add_row(
                "Resolution",
                f"{object.resolution} {object.get_meta_unit(object, 'resolution')}",
            )

        title: str = f"[wheat4][bold]#{object.number} - {object.name}[/bold] [italic]({object.abbreviation})[/italic]"

        return ReturnRender(renderable=table, title=title, subtitle=None)

    @staticmethod
    def get_bands_viz(object: Any, **kwargs: Any) -> ReturnRender:
        """Visualizer for `Bands`"""

        table = Table(row_styles=["yellow", "cyan"])

        title: str = f"{str(len(object))} Band"
        if len(object) > 0:
            title += "s"

        table.add_column("Number", justify="right")
        table.add_column("Abbreviation", justify="left")
        table.add_column("Name")
        table.add_column(
            f"Resolution ({object.get_meta_unit(list(object.__dict__.values())[0],'resolution')})",
            justify="right",
        )
        table.add_column(
            f"Wavelength ({object.get_meta_unit(list(object.__dict__.values())[0],'wavelength')})",
            justify="right",
        )
        table.add_column("Sensor", justify="right")

        for band in object:
            table.add_row(
                str(band.number),
                str(band.abbreviation),
                str(band.name),
                str(band.resolution),
                str(band.wavelength),
                str(band.sensor),
            )

        return ReturnRender(renderable=table, title=title, subtitle=None)

    @staticmethod
    def get_platform_viz(
        object: Any, show_description: bool, **kwargs: Any
    ) -> ReturnRender:
        """Visualizer for `Platform`"""

        NO_PRINT_ATTRIBUTES: Final[List[str]] = [
            "description",
            "var_name",
            "abbreviation",
            "name",
            "bands",
            "data_source",
            "number_bands",
        ]

        table = Table(
            show_header=False,
            show_lines=False,
            show_edge=False,
            expand=True,
            row_styles=["", "dim"],
        )

        table.add_column(justify="right", width=30)
        table.add_column(justify="left", width=30)

        field_values: Dict[str, str] = {
            f.name: str(f.default)
            for f in fields(object)
            if f.name not in NO_PRINT_ATTRIBUTES
        }
        field_values = dict(sorted(field_values.items(), key=lambda kv: kv[0]))

        for name, value in field_values.items():

            table.add_row(
                name.capitalize().replace("_", " "),
                f"{value} {object.get_meta_unit(object,name)}",
            )

        description: Text = Text("")
        if show_description:
            description = Text(object.description, justify="full", end="\n\n")

        band_table: ReturnRender = InfoVisualizers.get_bands_viz(object.bands)

        group: Group = Group(table, band_table.renderable, description, fit=False)

        if object.data_source:
            subtitle: str = f"[blue underline][link={object.data_source}]Source[/link]"

        title: str = f"[wheat4][bold]🛰️ {object.name}[/bold] [italic]({object.abbreviation})[/italic]"

        return ReturnRender(renderable=group, title=title, subtitle=subtitle)



from dataclasses import dataclass
from enum import Enum
from enum import auto
from typing import Any
from typing import List
from typing import Optional
from typing import cast

from eoplatform.composites.safe_eval import safe_eval  # type: ignore[attr-defined]
from eoplatform.composites.visualizers import CompositeVisualizers
from eoplatform.console import console
from eoplatform.shared_classes import ReturnRender
from rich.panel import Panel


class CompositeType(Enum):
    VEGETATION = auto()
    BURN = auto()
    WATER = auto()
    SNOW = auto()
    DROUGHT = auto()
    URBAN = auto()
    KERNEL = auto()
    NONE = auto()


@dataclass
class Composite:
    abbreviation: str
    formula: str
    name: str
    reference: str
    description: Optional[str]
    type: CompositeType
    bands: List[str]

    def info(self) -> None:

        render: ReturnRender = CompositeVisualizers.get_composite_viz(self)

        console.print(
            Panel(
                render.renderable,
                expand=False,
                title=render.title,
                subtitle=render.subtitle,
            )
        )

        return None

    def create(self, **kwargs: Any) -> Any:

        for band in self.bands:
            if band not in kwargs:
                raise ValueError(f"{band} must be given as a kwarg")

        return safe_eval(self.formula, **kwargs)

    def __str__(self) -> str:
        self.info()
        return ""



from eoplatform.console import console


# from eoplatform.platforms.Landsat8.downloader import Downloader as L8Downloader


def download(name: str) -> None:
    console.print(
        "[red bold encircle]:x:  downloading not yet implemented. Stay tuned!"
    )
    return



from typing import Optional
from typing import cast

from eoplatform.console import console
from eoplatform.platforms.classes import Bands
from eoplatform.platforms.classes import Platform
from eoplatform.platforms.factory import generate_platform


def info(name: str, only_bands: bool = False, show_description: bool = True) -> None:

    platform: Optional[Platform] = generate_platform(name.upper().strip())

    if platform is None:

        console.print(
            f"[red bold encircle]:x:  '{name}' platform not supported\n[/][yellow]Check your spelling and try again"
        )
        return

    if only_bands:
        cast(Bands, platform.bands).info()
        return

    platform.info(show_description=show_description)



from eoplatform.composites.factory import find_composite_names
from eoplatform.composites.interface import info as composite_info
from eoplatform.console import console
from eoplatform.platforms.factory import find_platform_names
from eoplatform.platforms.interface import info as platform_info


def info(name: str, only_bands: bool = False, show_description: bool = True) -> None:

    name = name.upper().strip()

    if name in find_platform_names():
        platform_info(
            name=name, only_bands=only_bands, show_description=show_description
        )

    elif name in find_composite_names():
        composite_info(name=name)

    else:
        console.print(
            f"[red bold encircle]:x:  '{name}' not recognized\n[/][yellow]Check your spelling and try again"
        )

    return None



from typing import Optional

from eoplatform.composites.classes import Composite
from eoplatform.composites.factory import generate_composite
from eoplatform.console import console


def info(name: str) -> None:

    composite: Optional[Composite] = generate_composite(name.upper().strip())

    if not composite:
        console.print(
            f"[red bold encircle]:x:  '{name}' composite not supported\n[/][yellow]Check your spelling and try again"
        )
        return

    composite.info()



from dataclasses import dataclass
from enum import Enum
from enum import auto
from typing import Optional

from rich.console import RenderableType


@dataclass
class ReturnRender:
    """Dataclass of visualization options dervied from `Band`, `Bands`, and `Platform`"""

    renderable: RenderableType
    title: Optional[str]
    subtitle: Optional[str]


class AutoImport(Enum):
    YES = auto()
    NO = auto()


class AutoImportControl:
    def __init__(self) -> None:
        self._status: AutoImport = AutoImport.YES

    @property
    def status(self) -> AutoImport:
        return self._status

    def set_status(self, status: AutoImport) -> None:

        if not isinstance(status, AutoImport):
            raise ValueError("Incorrect value ", status)

        self._status = status

        return None



from eoplatform.auto_import import auto_import
from eoplatform.composites.factory import generate_all_composites
from eoplatform.shared_classes import AutoImport


if auto_import.status == AutoImport.YES:
    for composite in generate_all_composites():
        locals()[composite.abbreviation] = composite



from dataclasses import field
from dataclasses import make_dataclass
from functools import lru_cache
import json
from pathlib import Path
from typing import Any
from typing import Dict
from typing import Final
from typing import Generator
from typing import List
from typing import Optional
from typing import Tuple
from typing import Union
from typing import cast

from eoplatform.composites.classes import Composite
from eoplatform.composites.classes import CompositeType
from eoplatform.utils import COMPOSITES_DIR


SEARCH_PATTERN: Final[str] = "**/*.json"


def generate_all_composites() -> Generator[Composite, None, None]:
    for name in find_composite_names():
        platform: Optional[Composite] = generate_composite(str(name))

        if not platform:
            break

        yield platform


@lru_cache(maxsize=5)
def generate_composite(name: str) -> Optional[Composite]:

    target_composite: Path
    for composite in _find_composite_files():
        if composite.stem.upper() == name.upper():
            target_composite = composite
            break
    else:
        return None

    composite_data: Dict[str, Union[str, int]] = _get_composite_data(target_composite)

    return _produce_composite(composite_data)


@lru_cache(maxsize=2)
def find_composite_names() -> List[str]:
    return [f.stem.upper() for f in COMPOSITES_DIR.glob(SEARCH_PATTERN)]


@lru_cache(maxsize=2)
def _find_composite_files() -> List[Path]:
    return [f for f in COMPOSITES_DIR.glob(SEARCH_PATTERN)]


@lru_cache(maxsize=10)
def _get_composite_data(composite_path: Path) -> Dict[str, Union[str, int]]:

    with open(str(composite_path), "r") as file:
        data: Dict[str, Union[str, int]] = json.load(file)

    return data


def _produce_composite(composite_dict: Dict[str, Any]) -> Composite:

    composite_type_str: str = (
        cast(str, composite_dict.pop("type", "NONE")).upper().strip()
    )
    if composite_type_str not in CompositeType.__members__.keys():

        raise ValueError(f"Incorrect composite type {composite_type_str}")
    composite_type: CompositeType = CompositeType[composite_type_str]

    composite_fields: List[Tuple[str, type, Any]] = [
        (
            k,
            cast(type, Any),
            field(),
        )
        for k in composite_dict.keys()
    ]

    composite_docstring: str = f"""{composite_dict.get("name")} ({composite_dict.get("abbreviation")})
        Type: {composite_type.name.capitalize()}

        Formula:
        `{composite_dict.get("formula")}`

        Bands:
        {', '.join(list(cast(List[str],composite_dict.get("bands", []))))}

        
        {composite_dict.get("description")}

        {composite_dict.get("reference")}
        """

    composite_template: type = make_dataclass(
        "Composite",
        composite_fields,
        bases=(Composite,),
        namespace={"__doc__": composite_docstring},
    )

    composite: Composite = composite_template(**composite_dict, type=composite_type)

    return composite



import typer

from eoplatform.main import info
from eoplatform.platforms.download.downloadControl import download


app = typer.Typer()


@app.command("info")
def info_controller(
    name: str,
    only_bands: bool = typer.Option(False, "--only-bands /", "-b /"),
    show_description: bool = typer.Option(True, " /--no-description", " /-nd"),
) -> None:

    info(
        name=name.upper().strip(),
        only_bands=only_bands,
        show_description=show_description,
    )

    return None


@app.command("download")
def platform_download(name: str) -> None:

    download(name=name)
    return


if __name__ == "__main__":
    app()



from importlib import resources
from pathlib import Path
from typing import Final


with resources.path("eoplatform", "__main__.py") as file:
    ROOT_DIR: Final[Path] = file.parent

PLATFORMS_DIR: Final[Path] = ROOT_DIR / "data" / "platforms"

COMPOSITES_DIR: Final[Path] = ROOT_DIR / "data" / "composites"



from dataclasses import dataclass
from dataclasses import field
from dataclasses import make_dataclass
from functools import lru_cache
import json
from pathlib import Path
from typing import Any
from typing import Dict
from typing import Final
from typing import Generator
from typing import List
from typing import Mapping
from typing import Optional
from typing import Tuple
from typing import Union
from typing import cast

from eoplatform.platforms.classes import Band
from eoplatform.platforms.classes import Bands
from eoplatform.platforms.classes import Platform
from eoplatform.utils import PLATFORMS_DIR


SEARCH_PATTERN: Final[str] = "**/*.json"


@dataclass
class PlatformNode:
    key: str
    value: Union[str, int, float, Dict[str, Any]]
    meta: Optional[Dict[str, Any]]


def generate_all_platforms() -> Generator[Platform, None, None]:
    for name in find_platform_names():
        platform: Optional[Platform] = generate_platform(str(name))

        if not platform:
            break

        yield platform


@lru_cache(maxsize=5)
def generate_platform(name: str) -> Optional[Platform]:

    target_platform: Path
    for platform in _find_platform_files():
        if platform.stem.upper() == name.upper():
            target_platform = platform
            break
    else:
        return None

    platform_data: Dict[str, Union[str, int]] = _get_platform_data(target_platform)

    return _produce_platform(platform_data)


@lru_cache(maxsize=2)
def _find_platform_files() -> List[Path]:

    return [f for f in PLATFORMS_DIR.glob(SEARCH_PATTERN)]


@lru_cache(maxsize=2)
def find_platform_names() -> List[str]:

    return [f.stem.upper() for f in PLATFORMS_DIR.glob(SEARCH_PATTERN)]


@lru_cache(maxsize=10)
def _get_platform_data(platform_path: Path) -> Dict[str, Union[str, int]]:

    with open(str(platform_path), "r") as file:
        data: Dict[str, Union[str, int]] = json.load(file)

    return data


def _produce_platform(platform_dict: Dict[str, Any]) -> Platform:

    platform_bands = platform_dict.pop("bands")

    platform_nodes: List[PlatformNode] = [
        PlatformNode(
            key=k,
            value=v.get("value", v) if isinstance(v, dict) else v,
            meta=v.get("meta", {}) if isinstance(v, dict) else {},
        )
        for k, v in platform_dict.items()
    ]

    platform_fields: List[Tuple[str, type, Any]] = [
        (
            n.key,
            cast(type, Any),
            field(default=n.value, metadata={**cast(Mapping[str, Any], n.meta)}),
        )
        for n in platform_nodes
    ]

    if platform_bands:
        bands_template: type = make_dataclass(
            "Bands",
            [
                (b["abbreviation"], Band, field(default=cast(Any, Band(**b))))
                for b in platform_bands
            ],
            bases=(Bands,),
        )
        platform_fields.append(("bands", Bands, field(default=bands_template())))

    platform_template: type = make_dataclass(
        "Platforms", platform_fields, bases=(Platform,)
    )

    platform: Platform = platform_template()

    return platform



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['eoplatform',
 'eoplatform.composites',
 'eoplatform.metadata',
 'eoplatform.platforms',
 'eoplatform.platforms.download',
 'eoplatform.platforms.info']

package_data = \
{'': ['*'], 'eoplatform': ['data/composites/*', 'data/platforms/*']}

install_requires = \
['numpy>=1.21.3,<2.0.0', 'rich>=10.12.0,<11.0.0', 'typer[all]>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['eoplatform = eoplatform.cli:app']}

setup_kwargs = {
    'name': 'eoplatform',
    'version': '0.2.4',
    'description': 'Earth Observation made easy.',
    'long_description': '<br/>\n<p align="center">\n  <a href="https://github.com/mtralka/EOPlatform">\n    <img src="images/logo.jpg" alt="EOP Logo" width="300" height="300">\n  </a>\n\n  <h3 align="center">An Earth Observation Platform</h3>\n\n  <p align="center">\n    Earth Observation made easy. \n    <br/>\n    <br/>\n    <a href="https://github.com/mtralka/EOPlatform/issues">Report Bug</a>\n    |\n    <a href="https://github.com/mtralka/EOPlatform/issues">Request Feature</a>\n  </p>\n</p>\n\n![Downloads](https://img.shields.io/github/downloads/mtralka/EOPlatform/total) ![Forks](https://img.shields.io/github/forks/mtralka/EOPlatform?style=social) ![Stargazers](https://img.shields.io/github/stars/mtralka/EOPlatform?style=social) <br/> ![Issues](https://img.shields.io/github/issues/mtralka/EOPlatform) ![License](https://img.shields.io/github/license/mtralka/EOPlatform) [![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black) ![mypy](https://img.shields.io/badge/mypy-checked-brightgreen)\n\n## About\n\n*eoplatform* is a Python package that aims to simplify Remote Sensing Earth Observation by providing actionable information on a wide swath of RS platforms and provide a simple API for downloading and visualizing RS imagery. Made for scientists, educators, and hobbiests alike.\n\n* Easy access to information on RS platforms\n* `metadata` module for extracting platform metadata\n  * supports `.txt` and `.xml` files\n* `composites` modules for creating and learning about 91 RS band composites\n\nComing soon:\n\n* Data downloading\n  * Landsat 8\n  * Sentinel-2\n* Raster tools\n  * Raster IO functions\n\n### Installation\n\n*eoplatform* can be installed by running `pip install eoplatform`. It requires Python 3.8 or above to run.\n\n#### *eoplatform* development is ongoing -\n\nupgrade to the latest version by running:\n`pip install eoplatform -U`\n\nInstall the latest dev version with:\n`pip install git+https://github.com/mtralka/EOPlatform/eoplatform`\n\n### Usage\n\n*eoplatform* is accessible through the command line (CLI) and as a module import.\n\n### Querying information\n\n#### ...through CLI\n\n`NAME` is any supported composite or platform (case-insensitive)\n\n```sh\nUsage: eoplatform info [OPTIONS] NAME\n\nArguments:\n  NAME  [required]\n\nOptions:\n  -b, --only-bands\n   / -nd, --no-description  [default: True]\n  --help                    Show this message and exit.\n```\n\nEX:\n\n**Show all information on `landsat8`**\n\n```sh\neoplatform info landsat8\n```\n\n![EOPlatform Landsat8 info example](images/eoplatform-info-landsat8.PNG)\n\n**Show only `landsat8` bands**\n\n```sh\neoplatform info landsat8 -b\n```\n\n**Show information on composite `NDVI`**\n\n```sh\neoplatform info ndvi\n```\n\n![EOPlatform NDVI info example](images/eoplatform-composite-info-ndvi.PNG)\n\n#### ...through imports\n\nYou can search through the `eoplatform` module\n\n```python\nimport eoplatform as eop\n\neop.info("Landsat8")  # case insensitive\neop.info("NDVI)\n```\n\nOr import your exact platform/composite\n\n```python\nfrom eoplatform.platforms import landsat8\nfrom eoplatform.composites import NDVI\n\nlandsat8.info()  # OR print(landsat8)\nlandsat8.bands.info()\nlandsat8.bands.RED.info()\nNDVI.info()  # or print(NDVI)\n```\n\n### Band composites\n\nImportable through `eoplatforms.composites` or searchable (shown above) from `eoplatform.info()`\n\nSee [implemented composite data dir](/eoplatform/data/composites) for the exact implemented composites (inspired by [awesome spectral indices](https://github.com/davemlz/awesome-spectral-indices))\n\n#### Composite information\n\n```python\nfrom eoplatform.composites import NDVI  # DVI, etc\n\nNDVI.info()\n```\n\n#### Creating composite\n\nComposites bands must be passed in as keyword arguments assigned to NumPy arrays.\n\n```python\nfrom eoplatform.composites import NDVI\n\nred_array: np.ndarray = ...\nnir_array: np.ndarray = ...\n\nndvi: np.ndarray = NDVI.create(NIR=nir_array, RED=red_array)\n```\n\n### Metadata extraction\n\nSupports `.txt` and `.xml` files through `extract_XML_metadata` and `extract_TXT_metadata`.\n\n```python\nfrom eoplatform.metadata import extract_XML_metadata\n\nfile_path: str = ...\ntarget_attributes: List[str] = ...\n\nvalues: Dict[str, str] = extract_XML_metadata(file_path, target_attributes)\n```\n\n## Adding platforms / composites\n\nPlatforms and composites are auto-generated from `/data/(composites,platforms)`. To add platforms or composites, simply create a new json file in the desired directory.\n\n### Platform\n\nPlatforms must have:\n\n```python\nabbreviation: str\nname: str\n```\n\n#### Example platform JSON\n\nStandard key with value:\n\n```json\n"abbreviation": "L8",\n//snip\n```\n\nKey with value and metadata:\n\n```json\n"altitude": {\n    "meta": {\n      "unit": "km"\n    },\n    "value": 705\n  }\n//snip\n```\n\n#### Bands standard for Platform key `bands`\n\nPlatform key for `bands` must be a list of following the `band` standard. EX:\n\n```json\n"bands": [\n    {\n      "abbreviation": "CAER",\n      "description":"",\n      "name": "Coastal Aerosol",\n      "number": 1,\n      "resolution": "30",\n      "sensor": "OLI",\n      "wavelength": "0.433-0.453"\n    },\n  //snip\n]\n```\n\n#### Band standard for el in `bands`\n\nEach element to `bands` is a `band`. Each band must have:\n\n```python\nnumber: int\nname: str\nabbreviation: str\n```\n\n### Composites\n\nComposites must have:\n\n```python\nabbreviation: str\nformula: str\nname: str\nreference: str\ntype: str  # matching CompositeType\nbands: List[str]\n```\n\n#### Example JSON Composite\n\n```json\n{\n  "abbreviation": "ARI",\n  "bands": [\n    "GREEN",\n    "VRE1"\n  ],\n  "description": "",\n  "formula": "(1 / GREEN) - (1 / VRE1)",\n  "name": "Anthocyanin Reflectance Index",\n  "reference": "https://doi.org/10.1562/0031-8655(2001)074%3C0038:OPANEO%3E2.0.CO;2",\n  "type": "vegetation"\n}\n\n```\n\n#### Composite Types\n\nPossible values for composite `type` key\n\n* VEGETATION\n* BURN\n* WATER\n* SNOW\n* DROUGHT\n* URBAN\n* KERNEL\n* NONE\n\n**For both platforms and composites, all other attributes will be dynamically rendered and shown in `info`**\n\n## Roadmap\n\nSee the [open issues](https://github.com/mtralka/EOPlatform/issues) for a list of proposed features (and known issues).\n\n* download support\n\n## Contributing\n\nContributions are welcome. Currently, *eoplatform* is undergoing rapid development and contribution opportunities may be scarce.\n\n* If you have suggestions for adding or removing features, feel free to [open an issue](https://github.com/mtralka/EOPlatform/issues/new) to discuss it, or directly create a pull request with the proposed changes.\n* Create individual PR for each suggestion.\n* Use pre-commit hooks - `pre-commit install`\n* Code style is `black`, `mypy --strict`\n\n## License\n\nDistributed under the GNU GPL-3.0 License. See [LICENSE](https://github.com/mtralka/EOPlatform/blob/main/LICENSE.md) for more information.\n\n## Built With\n\n* [Rich](https://github.com/willmcgugan/rich)\n* [Typer](https://github.com/tiangolo/typer)\n* [Numpy](https://numpy.org/)\n* [awesome spectral indices](https://github.com/davemlz/awesome-spectral-indices)\n\n## Authors\n\n* [**Matthew Tralka**](https://github.com/mtralka/)\n',
    'author': 'Matthew Tralka',
    'author_email': 'matthew@tralka.xyz',
    'maintainer': 'Matthew Tralka',
    'maintainer_email': 'matthew@tralka.xyz',
    'url': 'https://github.com/mtralka/EOPlatform',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)



from eoplatform.auto_import import auto_import
from eoplatform.platforms.factory import generate_all_platforms
from eoplatform.shared_classes import AutoImport


if auto_import.status == AutoImport.YES:
    for platform in generate_all_platforms():
        locals()[platform.var_name] = platform
